﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OperatorsReview
{
    public class Pracownik
    {
        public int PracownikId { get; set; }
        public string Login { get; set; }
        public string Imie { get; set; }
        public string Nazwisko { get; set; }
        public DateTime DataZatrudnienia { get; set; }
        public Oddzial Oddzial { get; set; }
    }

    public class Oddzial
    {
        public Oddzial()
        {
            Pracownicy = new List<Pracownik>();
        }
        public int OddzialId { get; set; }
        public string Nazwa { get; set; }
        public IList<Pracownik> Pracownicy { get; set; }
    }

    public class Repository
    {
        List<Pracownik> pracownicy = new List<Pracownik>();
        List<Oddzial> oddzialy = new List<Oddzial>();

        public Repository()
        {
            var o1 = new Oddzial { OddzialId = 1, Nazwa = "Marketing" };
            var o2 = new Oddzial { OddzialId = 2, Nazwa = "Handlowy" };
            var o3 = new Oddzial { OddzialId = 3, Nazwa = "Produkcja" };

            var p1 = new Pracownik { PracownikId = 1, Login = "M01", Imie = "J1", Nazwisko = "K1", DataZatrudnienia = new DateTime(2009, 1, 11), Oddzial = o1 };
            var p2 = new Pracownik { PracownikId = 2, Login = "M02", Imie = "J2", Nazwisko = "K2", DataZatrudnienia = new DateTime(2011, 2, 11), Oddzial = o1 };
            var p3 = new Pracownik { PracownikId = 3, Login = "X01", Imie = "J1", Nazwisko = "K3", DataZatrudnienia = new DateTime(2012, 3, 11), Oddzial = o2 };
            var p4 = new Pracownik { PracownikId = 4, Login = "X02", Imie = "J2", Nazwisko = "L4", DataZatrudnienia = new DateTime(2013, 4, 11), Oddzial = o2 };

            pracownicy.Add(p1);
            pracownicy.Add(p2);
            pracownicy.Add(p3);
            pracownicy.Add(p4);

            oddzialy.Add(o1);
            oddzialy.Add(o2);
            oddzialy.Add(o3);

            o1.Pracownicy.Add(p1);
            o1.Pracownicy.Add(p2);
            o2.Pracownicy.Add(p3);
            o2.Pracownicy.Add(p4);
        }

        public List<Pracownik> GetPracownicy()
        {
            return this.pracownicy;
        }

        public List<Oddzial> GetOddzialy()
        {
            return this.oddzialy;
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            var repository = new Repository();

            // let keyword
            var result1 =
                from p in repository.GetPracownicy()
                let loginLower = p.Login.ToLower()
                where loginLower.StartsWith("m")
                select loginLower;

            // grouping
            var result21 =
                from p in repository.GetPracownicy()
                group p by p.Oddzial.OddzialId;

            var result22 =
                from p in repository.GetPracownicy()
                group p by p.Oddzial into pracownicyWOddziale
                where pracownicyWOddziale.Count() > 1
                orderby pracownicyWOddziale.Key.Nazwa
                select pracownicyWOddziale;

            var result23 =
                from p in repository.GetPracownicy()
                group p by new { OddzialId = p.Oddzial.OddzialId, FirstLetter = p.Nazwisko[0].ToString() } into pracownicyWOddziale
                where pracownicyWOddziale.Count()>1
                select new 
                {
                    Oddzial = pracownicyWOddziale.Key.OddzialId,
                    Litera = pracownicyWOddziale.Key.FirstLetter,
                    Liczba = pracownicyWOddziale.Count()
                };

            var result24 = repository.GetPracownicy()
                .GroupBy( e=> new { OddzialId = e.Oddzial.OddzialId,
                                    FirstLetter = e.Nazwisko[0].ToString() })
                .Where( g=>g.Count() > 1)
                .Select ( p=> new { Oddzial = p.Key.OddzialId,
                                    Litera = p.Key.FirstLetter,
                                    Liczba = p.Count()
                                  });

            // nested queries
            var result31 =
                from p in repository.GetPracownicy()
                where p.Oddzial.OddzialId ==
                    (from o in repository.GetOddzialy()
                    where o.Nazwa == "Marketing"
                    select o).First().OddzialId
                select p;

            var result32 =
                from p in repository.GetPracownicy()
                select new
                {
                    Nazwisko = p.Nazwisko,
                    Imie = p.Imie,
                    Oddzial = (from o in repository.GetOddzialy()
                               where o.OddzialId == p.Oddzial.OddzialId
                               select o).First().Nazwa
                };

            // inner join
            var result41 =
                from p in repository.GetPracownicy()
                join o in repository.GetOddzialy()
                on p.Oddzial.OddzialId equals o.OddzialId
                select new { Nazwisko = p.Nazwisko, Imie = p.Imie, Oddzial = o.Nazwa };

            // left join
            var result42 =
                from o in repository.GetOddzialy()
                join p in repository.GetPracownicy()
                on o.OddzialId equals p.Oddzial.OddzialId
                into op
                select new { Oddzial = o.Nazwa, LiczbaPrac = op.Count() };
        }
    }
}
